@@ -76,45 +76,51 @@ int dialog_menu(const char *title, const char *prompt, int height, int width,
 		int menu_height, int item_no, const char *const *items)
 {
 	int i, x, y, cur_x, cur_y, box_x, box_y;
-	int key = 0, button = 0, choice = 0, scroll = 0, max_choice;
+	int key = 0, button = GTK_RESPONSE_OK, choice = 0, scroll = 0, max_choice;
 	WINDOW *dialog, *menu;
 
-	max_choice = MIN(menu_height, item_no);
-	
-	if(gnome_mode)
-	{
-		GtkWidget *w = gnome_dialog_new(title, 	
-			GNOME_STOCK_BUTTON_CANCEL, NULL);
-		GtkWidget *hbox;
-		GtkWidget *vbox;
+ 	max_choice = MIN(menu_height, item_no);
+
+	if (gnome_mode)	{
+		GtkWidget *w;
 		GtkWidget *but;
 		GtkWidget *first_button;
-		GtkWidget *butframe;
 		GtkWidget *butbox;
+		GtkWidget *sw, *viewport;
+ 
+		w = gtk_dialog_new_with_buttons (title,	NULL,
+				GTK_DIALOG_DESTROY_WITH_PARENT,
+				GTK_STOCK_CANCEL,
+				GTK_RESPONSE_CANCEL,
+				NULL);		
+                gtk_signal_connect (GTK_OBJECT (w), "response",
+                              GTK_SIGNAL_FUNC (esc_cancel), NULL);
+		gtk_window_set_position (GTK_WINDOW (w), GTK_WIN_POS_CENTER);
 		
+		label_autowrap (GTK_DIALOG(w)->vbox, prompt, width);
+
+		/*
+		 * Setup the containers.
+		 */
+		sw = gtk_scrolled_window_new (NULL, NULL);
+ 		gtk_box_pack_start_defaults (
+			GTK_BOX (GTK_DIALOG (w)->vbox), sw);
+		gtk_widget_show (sw);
+
+ 		viewport = gtk_viewport_new (NULL, NULL);
+		gtk_container_add (GTK_CONTAINER (sw), viewport);
+ 		gtk_widget_show (viewport);
+ 
+ 		butbox = gtk_vbox_new (FALSE, 0);
+ 		gtk_container_add (GTK_CONTAINER (viewport), butbox);
+ 		gtk_widget_show (butbox);
+
+		gtk_container_set_border_width (GTK_CONTAINER (butbox),
+						GNOME_PAD);
+		/*
+		 * Add the buttons.
+		 */
 		the_items = items;
-
-		gtk_signal_connect (GTK_OBJECT (w), "destroy",
-				    GTK_SIGNAL_FUNC (esc_cancel), NULL);
-
-		gnome_dialog_set_close(GNOME_DIALOG(w), TRUE);
-		gtk_window_set_title(GTK_WINDOW(w), title);
-		gnome_dialog_button_connect (GNOME_DIALOG (w), 0,
-					     GTK_SIGNAL_FUNC(cancel_callback),
-					     NULL);
-		
-		hbox = gtk_hbox_new(FALSE, 0);
-		vbox = gtk_vbox_new(FALSE, 0);
-		
-		label_autowrap(vbox, prompt, width);
-		
-		butbox=gtk_vbox_new(FALSE, 0);
-		butframe=gtk_frame_new("");
-		
-		gtk_container_set_border_width(GTK_CONTAINER(butframe), GNOME_PAD);
-		gtk_container_set_border_width(GTK_CONTAINER(butbox), GNOME_PAD);
-		gtk_frame_set_shadow_type(GTK_FRAME(butframe), GTK_SHADOW_ETCHED_IN);
-				
 		first_button = NULL;
 		
 		for(i=0; i< max_choice; i++)
@@ -134,16 +140,7 @@ int dialog_menu(const char *title, const char *prompt, int height, int width,
 			gtk_signal_connect(GTK_OBJECT(but), "clicked",
 			GTK_SIGNAL_FUNC(okayed), GUINT_TO_POINTER(i));
 		}
-		
-		gtk_container_add(GTK_CONTAINER(butframe), butbox);
-		
-		gtk_box_pack_start(GTK_BOX(vbox), butframe, TRUE, TRUE, 0);
-		gtk_box_pack_start(GTK_BOX(hbox), vbox, TRUE, TRUE, 0);
 
-		gtk_box_pack_start(GTK_BOX(GNOME_DIALOG(w)->vbox), 
-			hbox,
-			TRUE, TRUE, GNOME_PAD);
-		gtk_window_set_position(GTK_WINDOW(w), GTK_WIN_POS_CENTER);
 		gtk_widget_show_all(w);
 		if (first_button != NULL)
 			gtk_widget_grab_focus (first_button);
@@ -204,7 +201,7 @@ int dialog_menu(const char *title, const char *prompt, int height, int width,
 	tag_x = 0;
 	item_x = 0;
 	/* Find length of longest item in order to center menu */
-	for (i = 0; i < item_no; i++) {
+	for (i = 0; i < max_choice; i++) {
 		tag_x = MAX(tag_x,
 		    strlen(items[i * 2]) + strlen(items[i * 2 + 1]) + 2);
 		item_x = MAX(item_x, strlen(items[i * 2]));
@@ -213,7 +210,7 @@ int dialog_menu(const char *title, const char *prompt, int height, int width,
 	item_x = tag_x + item_x + 2;
 
 	/* Print the menu */
-	for (i = 0; i < max_choice; i++)
+	for (i = 0; i < item_no; i++)
 		print_item(menu, items[i * 2], items[i * 2 + 1],
 			   i, i == choice);
 	wnoutrefresh(menu);
@@ -376,17 +373,20 @@ int dialog_menu(const char *title, const char *prompt, int height, int width,
 			return -2;
 		case M_EVENT + 'o':	/* mouse enter... */
 		case M_EVENT + 'c':	/* use the code for toggling */
-			button = (key == M_EVENT + 'o');
+			if(key == M_EVENT + 'o')
+				button = GTK_RESPONSE_CANCEL;  
+			else
+				button = GTK_RESPONSE_OK; 
 		case ' ':
 		case TAB:
 		case KEY_LEFT:
 		case KEY_RIGHT:
-			if (!button) {
-				button = 1;	/* Indicates "Cancel" button is selected */
+			if (button == GTK_RESPONSE_OK) {
+				button = GTK_RESPONSE_CANCEL;/* Indicates "Cancel" button is selected */
 				print_button(dialog, "  OK  ", y, x, FALSE);
 				print_button(dialog, "Cancel", y, x + 14, TRUE);
 			} else {
-				button = 0;	/* Indicates "OK" button is selected */
+				button = GTK_RESPONSE_OK;/* Indicates "OK" button is selected */
 				print_button(dialog, "Cancel", y, x + 14, FALSE);
 				print_button(dialog, "  OK  ", y, x, TRUE);
 			}
@@ -394,7 +394,7 @@ int dialog_menu(const char *title, const char *prompt, int height, int width,
 			break;
 		case '\n':
 			delwin(dialog);
-			if (button == 1) return -2;
+			if (button == GTK_RESPONSE_CANCEL) return -2;
 			return scroll+choice;
 		case ESC:
 			break;
